# -*- coding: utf-8 -*-

import Manyo as mm
import os
################################
#      VanadiumCorrection      #
################################
class VanadiumCorrection(object):
    """
    Vanadium Correction
    Data Correction using White Vanadium or Monochromatic Vanadium
    """
    ################################
    def __init__(self, SampleData=None, WhiteVan=None, MonoVan=None, WhiteVanM=None):
        """
        Constructor
        
        @SampleData (ElementContainerMatrix) Sample data
        @WhiteVan (UInt4) White Vanadium run number
        @MonoVan (UInt4) Monochromatic Vanadium run number
        @WhiteVanM (UInt4) White Vanadium run number for Monochromatic Vanadium run
        """
        self.inst_code = 'SIK'
        self.SampleData=None  
        self.WV_run = None
        self.WV_EC = None
        self.MV_run = None
        self.MV_EC = None
        self.WVM_run = None
        self.WVM_EC = None

        self.PathToVanData = "./"
        self.EventDataPath = "../RawData/SIK10_081120000000/"
        self.DetectorInfoFile = "./DetectorInfo_16PSD2.xml"

        self.CommentHead = "Vanadium >> "
        self.DoFlag = False
        self.SetFlag = [0,0,0,0]

        self.PeakPosi_m1 = 0.0
        self.Lms = 0.0

        self.Set(SampleData, WhiteVan, MonoVan, WhiteVanM)


    ################################
    def Set(self, SampleData=None, WhiteVan=None, MonoVan=None, WhiteVanM=None):
        """
        Set Parameters
        Set Parameters for the process of Vanadium Correction
        @SampleData (ElementContainerMatrix) Sample data
        @WhiteVan (UInt4) White Vanadium run number
        @MonoVan (UInt4) Monochromatic Vanadium run number
        @WhiteVanM (UInt4) White Vanadium run number for Monochromatic Vanadium run
        """
        if SampleData!=None: # sample data ElementContainerMatrix import
            #if type(SampleData)==type(mm.ElementContainerMatrix()):
            if isinstance( SampleData, mm.ElementContainerMatrix ):
                self.SampleData=SampleData
                self.SetFlag[0]=2

        if WhiteVan!=None:   
            if type(WhiteVan)==type(0):
                fn = self._VanDataExist(WhiteVan,1)
                if fn!=None:
                    self.WV_run = int(WhiteVan)
                    self.WV_EC = self.LoadFromFile(fn)
                    self.SetFlag[1]=2
                else:
                    print self.CommentHead+" Warnning : There is no WhiteVan data of run number you inputted."
                    self.WV_run = int(WhiteVan)
                    self.WV_EC = None
                    self.SetFlag[1]=1
                                   
        if MonoVan!=None:
            if type(MonoVan)==type(0):
                fn = self._VanDataExist(MonoVan,2)
                if fn!=None:
                    self.MV_run = int(MonoVan)
                    self.MV_EC = self.LoadFromFile(fn)
                    self.SetFlag[2]=2
                else:
                    print self.CommentHead+" Warnning : There is no MonoVan data of run number you inputted."
                    self.MV_run = int(MonoVan)
                    self.MV_EC = None
                    self.SetFlag[2]=1

        if WhiteVanM!=None:
            if type(WhiteVanM)==type(0):
                fn = self._VanDataExist(WhiteVanM,3)
                if fn!=None:
                    self.WVM_run = int(WhiteVanM)
                    self.WVM_EC = self.LoadFromFile(fn)
                    self.SetFlag[3]=2
                else:
                    print self.CommentHead+" Warnning : There is no WhiteVan for MonoVan data of run number you inputted."
                    self.WVM_run = int(WhiteVanM)
                    self.WVM_EC = None
                    self.SetFlag[3]=1

    ################################
    def DoVanCorrection(self):
        """
        Do Vanadium Correction
        Do Vanadium Correction using White Vanadium or Monochromatic Vanadium
        """
        Flag = True
        
        if self.SampleData==None:
            print self.CommentHead+"Error : Sample data have never been set."
            Flag = False
        if (self.WV_EC==None)and(self.MV_EC==None):
            print self.CommentHead+"Error : Both Vanadium run number have never been set."
            Flag = False

        if Flag:                  # If there are Sample data and Vanadium 
            process_str = ""
                                  # if we have W_s, MV, W_mv
            if (self.WV_EC!=None)and(self.MV_EC!=None)and(self.WVM_EC!=None):
                index_WV = self.WV_EC.PutX()
                count_WV = self.WV_EC.PutY()
                index_MV = self.MV_EC.PutX()
                count_MV = self.MV_EC.PutY()
                index_WVM = self.WVM_EC.PutX()
                count_WVM = self.WVM_EC.PutY()
                if index_WV.size()!=index_MV.size(): 
                    print self.CommentHead+" Warning : Index of WhiteVan and one of MonoVan are different."
                index_v = []
                count_v = []
                for i in range(index_WV.size()): 
                    index_v.append( index_WV[i] )
                    count_v.append( count_WV[i]/count_WVM[i]*count_MV[i] )
                process_str = "WV+MV+WVM"

            elif (self.WV_EC!=None)and(self.MV_EC==None): # if we have only W_s
                index_v = self.WV_EC.PutX()
                count_v = self.WV_EC.PutY()
                process_str = "WV"
                
            else:                                        # if we have only MV
                index_v = self.MV_EC.PutX()
                count_v = self.MV_EC.PutY()
                process_str = "MV"

            print "[DoCorrection] len(count_v)=",len(count_v)
            for i in range(1,self.SampleData.PutTableSize()):  # Do correction
                for j in range(self.SampleData(i).PutTableSize()):
                    pixel_id = self.SampleData(i,j).PutHeaderPointer().PutInt4("PIXELID")
                    ccc = count_v[pixel_id]
                    if ccc==0.0:
                        ccc=1.0e-10
                    dammy = self.SampleData(i,j).MulMySelf(1.0/ccc)
            del dammy
            hd = self.SampleData.PutHeaderPointer()
            process = hd.PutStringVector("DATAPROCESSED")
            process.append(":VanCorr(" + process_str + ")")
            hd.OverWrite("DATAPROCESSED", process)
            self.DoFlag = True

        else :
            print self.CommentHead+" ALART!! : We CANNOT correct data by Vanadium data."

    ################################
    def PutCorrectedData(self):
        """
        Put corrected data
        Put data corrected by Vanadium
        @retval (ElementContainerMatrix) 
        """
        if self.DoFlag:
            return self.SampleData
        else :
            print self.CommentHead+" ALART!! : We CANNOT Do vanadium corrections."
            return self.SampleData

    ################################
    def _VanDataExist(self,runnum,which):
        """
        Check file path to Vanadium data
        Check file path to vanadium data and return its path
        @param runnum(Int4) Run Number for Vanadium
        @param which(Int4) 1:WhiteVan for sample, 2:MonoVan, 3:WhiteVan for MonoVan
        @retval (None or String)
        """
        DataName = self._ReturnVanDataFile(runnum,which)
        if DataName!="None":
            fn = self.PathToVanData + DataName
            if os.path.exists(fn):
                return fn
            else:
                print "There is no file name =",fn
                return None
        else:
            print "ReturnVanDataFile=None"
            return None

    ################################
    def SetEventDataToHistXMLFile(self, FileName):
        """
        Set XML file 
        Set XML file into this class for MakeVanadiumData
        @param (String) file name of XML information.
        """
        self.DetectorInfoFile=FileName

    ################################
    def PutWhiteVanEC(self):
        """
        Put ElementContainer for White Vanadium 
        Put ElementContainer for White Vanadium calculated for the vanadium corrections
        @retval (ElementContainer)  
        """
        if self.WV_EC==None:
            print "There is no data for White Vanadium."
            return -1
        else:
            return self.WV_EC

    ################################
    def PutMonoVanEC(self):
        """
        Put ElementContainer for Monochromatic Vanadium 
        Put ElementContainer for Monochromatic Vanadium calculated for the vanadium corrections
        @retval (ElementContainer)  
        """
        if self.MV_EC==None:
            print "There is no data for Monochro Vanadium."
            return -1
        else:
            return self.MV_EC

    ################################
    def _ReturnVanDataFile(self,runnum,which):
        """
        Return data file name of analysed vanadium
        Get file name of analysed vanadium data from DB and return it in future
        @param runnum (Int) Runnumber
        @param which(Int) 1:WhiteVan, 2:MonoVan, 3:WhiteVan for MonoVan
        @retval string
        """
        stamp = ""
        st_runnum="%6d" % (runnum)
        if (which == 1)|(which == 3):
            stamp = "wvd"
        elif which == 2:
            stamp = "mvd"
        
        print "[ReturnVanDataFile] stamp=",stamp
        if stamp!="":
            ret = self.inst_code + st_runnum.replace(' ','0') + "." + stamp
            print ret
            return ret
        else:
            return "None"
        
    ################################
    def SetMonitorInfo(self,Lms):
        """
        Set the length between monitor 1 to sample
        
        @param Lms(Double) Length between monitor 1 to sample
        
        """
        self.Lms = Lms

    ################################
    def MakeVanData(self):
        """
        Make and save data for Vanadium correction
        Make and save data for Vanadium correction for White and Monochromatic
        """
        for which in range(1,3):
            if (self.SetFlag[which]==1):
                self._MakeVanadiumEC(which)
                self.SaveToFile(which)


    ################################
    def _MakeVanadiumEC(self,Which):
        """
        Make data for vanadium correction
        Make data for vanadium correction 
        @param Which (UInt4) 1: White Van, 2: Mono Van, 3: White Van for Mono van
        """
        import EventDataToHist as EVT
        import MonitorAnalysis as MON
        from DataReduct_SJY import Tof2Energy, Tof2EnergyWhite
        from sub1 import simplehistogram, ecsub, eccopy

        if Which==1:
            runnum = self.WV_run
        elif Which==2:
            runnum = self.MV_run
        elif Which==3:
            runnum = self.WVM_run
        else:
            return

        e2h = EVT.EventDataToHist(self.inst_code,runnum,debug=0)
        e2h.ImportDammyMonitor()
        ret = e2h.SetParamFile(self.DetectorInfoFile,0)
        if ret==-1:
            print "XML file is no correct."
            return
        e2h.SetPathToData(self.EventDataPath)
        e2h.MakeEventDataFileName()
        VanDat=e2h.PutElementContainers()

        MaxPixelId = e2h.EtoH.PutMaxOfPixelId()
        
        if Which==2:
            M_mv = MON.MonitorAnalysis(dat)
            M_mv.SearchPeakPosition(0)
        
            Ei, Ei_unit = M_mv.PutIncidentEnergy()
            delta_Ei, delta_Ei_unit = M_mv.PutIncidentEnergyWidth()
            Ei_ini = Ei-delta_Ei/2.0
            Ei_fin = Ei+delta_Ei/2.0
            self.PeakPosi_m1 = mon.PutPeakPosi(0)
            Tof2Energy(VanDat, self.Lms, self.PeakPosi_m1)
 
        else:
            Ei, Ei_unit = (0.0,"meV")
            delta_Ei, delta_Ei_unit = (0.0,"meV")
            Ei_ini = 20.0
            Ei_fin = 50.0
            print "Tof2Energy White--"
            Tof2EnergyWhite(VanDat, 0.0 )
        
        V_EC = mm.ElementContainer()
        list_pixelId=[]
        list_integ=[]
        list_dam_err=[]
        for i in range(MaxPixelId+1):
            list_pixelId.append(i)
            list_integ.append(0.0)
            list_dam_err.append(1.0)
        list_pixelId.append(MaxPixelId)
        
        if VanDat(0).PutHeaderPointer().PutString("TYPE")=="MONITOR":
            start_i = 1
        else:
            start_i = 0
        VanDat(1,0).Dump()
        print "Max of PixelID = ",MaxPixelId
        print "length of List_pixelId =",len(list_pixelId)
        for psd in range(start_i, VanDat.PutTableSize()):
            for pixel in range(VanDat(psd).PutTableSize()):
                pixel_id=VanDat(psd)(pixel).PutHeaderPointer().PutInt4("PIXELID")
                sh = simplehistogram(VanDat(psd)(pixel))
                try:
                    (d1,d2)=sh.integrate(ini=(Ei_ini),fin=(Ei_fin))
                    list_integ[pixel_id] = float(d1)
                    list_dam_err[pixel_id] = float(d2)
                except:
                    print "Errored Id =",pixel_id
                    return
                               
        for i in range(len(list_integ)):
            print list_integ[i]

        V_EC.Add("PixelId",list_pixelId)
        V_EC.Add("IntegMonVan",list_integ)
        V_EC.Add("DammyErr", list_dam_err)
        V_EC.SetKeys("PixelId","IntegMonVan","DammyErr")

        if Which==1:
            self.WV_EC = V_EC
        elif Which==2:
            self.MV_EC = V_EC
        else:
            self.WVM_EC = V_EC
        
        del VanDat, V_EC, list_pixelId, list_integ, list_dam_err
        
        

    ################################
    def SaveToFile(self, Which=0):
        """
        Save data for Vanadium corrections 
        Save data for Vanadium corrections with NeXus format
        @param Which (UInt4) 1: White Van, 2: Mono Van, 3: White Van for Mono van
        """
        runnum = None
        if Which==1:
            runnum = self.WV_run
        elif Which==2:
            runnum = self.MV_run
        elif Which==3:
            runnum = self.WVM_run

        if runnum==None:
            print "Vanadium infor not setting."
            return
        
        if self.DetectorInfoFile==None:
            print "DetectorInfo file is not set."
            return

        filename = self._ReturnVanDataFile(runnum,Which)
        
        fp = open(self.DetectorInfoFile,"r")
        XmlContents = fp.read()
        fp.close()

        w = mm.WriteNeXusFile( filename, "VanadiumCorrect", 1 )
        w.MakeOpenGroup( "Entry1", "NXentry" )
        w.MakeOpenGroup( "Vanadium", "NXdata" )
        w.WriteData( self.DetectorInfoFile, XmlContents )
        if Which==1:
            w.WriteData( "WhiteVan", self.WV_EC )
        elif Which==2:
            w.WriteData( "MonoVan", self.MV_EC )
        else:
            w.WriteData( "WhiteVan", self.WVM_EC )

        w.CloseGroup()
        w.CloseGroup()
        
        del w


    ################################
    def LoadFromFile(self, filename):
        """
        Load data from NeXus file
        Load data for Vanadium corrections from NeXus file
        @param filename (String) filename
        """
        if filename[-3:]=="wvd":
            tag = "WhiteVan"
        elif filename[-3:]=="mvd":
            tag = "MonoVan"
        else:
            print "Filename is not good for van data"
            return None

        rf = mm.ReadNeXusFile( filename )
        vs1 = rf.GetNextEntry()
        rf.OpenGroup( vs1[0], vs1[1] )
        
        vs2 = rf.GetNextEntry()
        rf.OpenGroup( vs2[0], vs2[1] )
        
        VanEC = rf.ReadElementContainer( tag )
        
        rf.CloseGroup()
        rf.CloseGroup()
        
        VanEC.SetKeys("PixelId","IntegMonVan","DammyErr")
        return VanEC
        
            



# -*- coding: utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        SliceViewer
# Purpose:
#
# Author:
#
# Created:     22/01/2016
# Copyright:   (c)
# Licence:
#-------------------------------------------------------------------------------

#######################################
#  SliceViewer
#######################################

import thread
import os, sys
import socket
import time
import threading
import Queue
import subprocess

class SliceViewer(object) :
    """
    プロッタのファサードクラス
    Python インタープリタ からプロッタを起動し
    制御するためのユーザインターフェイスを提供
    """
    # プロッタ通信クラスのインスタンスリスト
    plotList = []
    # plotList のロック用
    lck = thread.allocate_lock()

    #########################################
    def __init__(self,data=None, closeFlag = True):
        """
        コンストラクタ
        @param  data  プロッタに表示するデータ、省略可
        　　　　エレメントアレイまたはエレメントアレイのリスト
        @closeFlag   親プロセス終了時に、プロッタをクローズするか
        　　　　　　 否かのフラグ
        @retval 無し
        """
        self.plot = None

        # プロッタの作成
        self.NewPlot(data, closeFlag)

    #########################################
    def NewPlot(self, data=None, closeFlag = True):
        """
        プロッタを追加し、カレントプロッタとする
        @param  data  プロッタに表示するデータ、省略可
        　　　　エレメントアレイまたはエレメントアレイのリスト
        @param closeFlag  親プロセス終了時に、子プロセスを終了させるかどうかのフラグ
        @retval 無し
        """
        # 引数リストを作成
        pno = len(self.plotList) +1
        strPno = "%d" % pno

        if closeFlag == True:
            cflag = "True"
        else:
            cflag = "False"

        arg = ["SliceViewerMain.py", strPno, cflag]
        # プロセスID とプロッタ番号からポート番号を生成
        pxx = (os.getpid() % 500)*20
        portno = 10000+ pxx + pno

        # GUI通信クラスのインスタンス取得
        p = CommWx()

        # プロセス生成と通信接続に成功したなら
        if p.NewComm(portno, arg):
            # 旧カレントプロッタのアイコン化要求
            #self._IconizeCurrent()

            self.lck.acquire()
            # 追加したプロッタをリストへ追加
            #self.plotList.append(p)
            # ロックの解放
            self.lck.release()

            # 追加したプロッタをカレントプロッタとする。
            self.plot = p

            # データが指定されていたら
            if not data == None:
                # データを追加
                self.AddData(data)
        else:
            print "\nError!!  Cannot create a new plotter!!."

#######################################
#  User Command
#######################################
# コマンドヘルプ
    def help(self):
        print u"*---------------------------"
        print u"*** Slice Viewer Command ***"
        print u"*---------------------------"
        print u"+ setFocalPoint(x, y, z)"
        print u"+ setViewPoint(x, y, z)"
        print u"+ setDistance(dist)"
        print u"+ setAzimuth(az)"
        print u"+ setElevation(ele)"
        print u"+ setScalarRange(min, max)"
        print u"+ setAxisLength(lenX, lenY, lenZ)"
        print u"+ setGridScale(scaleX, scaleY, scaleZ)"
        print u"+ setOutputDirectory(directory)"
        print u"+ doSliceAx1(proc, pos, delta, ax2bin, ax3bin)"
        print u"  - proc:"
        print u"      0: Draw on main view"
        print u"      1: Draw by M2Plot"
        print u"      2: Save as Image (should setOutputDirectory)"
        print u"      3: Save as ECA (should setOutputDirectory)"
        print u"  - pos: position of plane"  
        print u"  - delta: thickness of plane"
        print u"  - ax2bin, ax3bin: bin for each direction"
        print u"+ doSliceAx2(proc, pos, delta, ax3bin, ax1bin)"
        print u"  - proc:"
        print u"      0: Draw on main view"
        print u"      1: Draw by M2Plot"
        print u"      2: Save as Image (should setOutputDirectory)"
        print u"      3: Save as ECA (should setOutputDirectory)"
        print u"  - pos: position of plane"  
        print u"  - delta: thickness of plane"
        print u"  - ax1bin, ax3bin: bin for each direction"
        print u"+ doSliceAx3(proc, pos, delta, ax1bin, ax2bin)"
        print u"  - proc:"
        print u"      0: Draw on main view"
        print u"      1: Draw by M2Plot"
        print u"      2: Save as Image (should setOutputDirectory)"
        print u"      3: Save as ECA (should setOutputDirectory)"
        print u"  - pos: position of plane"  
        print u"  - delta: thickness of plane"
        print u"  - ax1bin, ax2bin: bin for each direction"
        print u"+ doFreeSlice(proc, org[0], org[1], org[2], x[0], x[1], x[2], "
        print u"              y[0], y[1], y[2], xbin, ybin, xrange[0], xrange[1], yrange[0], yrange[1], delta)"
        print u"  - proc:"
        print u"      0: Draw on main view"
        print u"      1: Draw by M2Plot"
        print u"      2: Save as Image (should setOutputDirectory)"
        print u"      3: Save as ECA (should setOutputDirectory)"
        print u"  - org: Origin of plane"  
        print u"  - x: point on x-axis"
        print u"  - y: point on y-axis"
        print u"  - xbin, ybin: bin for each direction"
        print u"  - xrange, yrange: view range for each direction"
        print u"  - delta: thickness of plane"


# 注視点を指定
    def setFocalPoint(self, x, y, z):
        strCmd = "setFocalPoint %s %s %s\n" % (str(x), str(y), str(z))
        self.plot.PutCommand(strCmd)

# 注視点を指定
    def setViewPoint(self, x, y, z):
        strCmd = "setViewPoint %s %s %s\n" % (str(x), str(y), str(z))
        self.plot.PutCommand(strCmd)
        
# 視点の距離を指定
    def setDistance(self, dist):
        strCmd = "setDistance %s\n" % str(dist)
        self.plot.PutCommand(strCmd)

# 方位角を指定
    def setAzimuth(self, az):
        strCmd = "setAzimuth %s\n" % str(az)
        self.plot.PutCommand(strCmd)
        
# 視点の仰角を指定
    def setElevation(self, ele):
        strCmd = "setElevation %s\n" % str(ele)
        self.plot.PutCommand(strCmd)        

# set scalar range
    def setScalarRange(self, min, max):
        strCmd = "setScalarRange %s %s\n" %(str(min), str(max))
        self.plot.PutCommand(strCmd)

# set axis length
    def setAxisLength(self, lenX, lenY, lenZ):
        strCmd = "setAxisLength %s %s %s\n" %(str(lenX), str(lenY), str(lenZ))
        self.plot.PutCommand(strCmd)

# set grid scale
    def setGridScale(self, scaleX, scaleY, scaleZ):
        strCmd = "setGridScale %s %s %s\n" %(str(scaleX), str(scaleY), str(scaleZ))
        self.plot.PutCommand(strCmd)

# set output directory
    def setOutputDirectory(self, directory):
        strCmd = "setOutputDirectory %s\n" %(str(directory))
        self.plot.PutCommand(strCmd)

# basic slice Ax1
    def doSliceAx1(self, proc, pos, delta, ax2bin, ax3bin):
        strCmd = "doSliceAx1 %s %s %s %s %s\n" %(str(proc), str(pos), \
                  str(delta), str(ax2bin), str(ax3bin))
        self.plot.PutCommand(strCmd)

# basic slice Ax2
    def doSliceAx2(self, proc, pos, delta, ax3bin, ax1bin):
        strCmd = "doSliceAx2 %s %s %s %s %s\n" %(str(proc), str(pos), \
                  str(delta), str(ax3bin), str(ax1bin))
        self.plot.PutCommand(strCmd)

# basic slice Ax3
    def doSliceAx3(self, proc, pos, delta, ax1bin, ax2bin):
        strCmd = "doSliceAx3 %s %s %s %s %s\n" %(str(proc), str(pos), \
                  str(delta), str(ax1bin), str(ax2bin))
        self.plot.PutCommand(strCmd)

    def doFreeSlice(self, proc, org1, org2, org3, x1, x2, x3, y1, y2, y3, \
                 xbin, ybin, xrange1, xrange2, yrange1, yrange2, thickness):
        strCmd = "doFreeSlice %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s\n" %(str(proc), str(org1), \
                                                                                      str(org2), str(org3), str(x1), \
                                                                                      str(x2), str(x3), str(y1), str(y2), str(y3), \
                                                                                      str(xbin), str(ybin), str(xrange1), \
                  str(xrange2), str(yrange1), str(yrange2), str(thickness))

        self.plot.PutCommand(strCmd)

# set data [inamura 170227]-->
    def setData(self, proc, ecm, title="Empty", xkey="-", ykey="-", zkey="-"):
        from vis.UtilPlot import TempFile
        import Manyo
        TMPF = TempFile()
        temppath = TMPF.GetTempName(".srlz")
        if temppath==None:
            print "Error Error Error"
            return
        crw = Manyo.WriteSerializationFileBinary(temppath)
        crw.Save(ecm)
        del crw
        
        title = title.strip().replace(" ","_")
        strCmd = "setData %s %s %s %s %s\n"%(temppath,title,xkey,ykey,zkey)
        #print "command=",strCmd
        ret = self.plot.PutCommand(strCmd)
        if ret==None:
            return False
        else:
            return True

    def setAxesTitles(self, xtitle="X", ytitle="Y",  ztitle="Z"):
        strCmd = "setAxesTitles %s %s %s\n"%(xtitle,ytitle,ztitle)
        ret = self.plot.PutCommand(strCmd)
        if ret==None:
            return False
        else:
            return True
#<--[inamura 170227]
        
#[inamura 160920]-->
#########################################
from vis.UtilPlot import AcqPath 
def SearchPath(fname,isFile=True):
    """
    フルパスの探索
    @param  fname  　ファイル名
    @retval ファイルのフルパス
    """ 
    # カレントディレクトリを取得
    fpath = os.path.join(os.getcwd(), fname)
    # カレントディレクトリに目的ファイルがあるか
    if isFile:
        if os.path.isfile(fpath):
            # フルパスを返す
            return (os.getcwd(),fpath)
    else:
        if os.path.isdir(fpath):
            return fpath
    # 環境変数に指定されたディレクトリを取得
    ap = AcqPath()
    paths = [ap.GetSysExp(), ap.GetSysAnal(), ap.GetSysVis()]
    p_path = os.environ.get('PYTHONPATH')
    if p_path!=None:
        p_path_list = p_path.split(os.pathsep)
        for pp in p_path_list:
            paths.append(pp)
            paths.append( os.path.join(pp,"vis","SliceViewer3D") )
                
    # 全PYTHONPATHディレクトリを探索
    for path in paths:
        fpath = os.path.normpath(os.path.join(path,fname))
        # ディレクトリに目的ファイルがあるか
        if isFile:
            if os.path.isfile(fpath):
                # フルパスを返す
                return (path,fpath)
        else:
            if os.path.isdir(fpath):
                return fpath

    # 目的ファイルが見つからなかった
    return None
#########################################
view_path = SearchPath( "View", isFile=False )
if view_path!=None:
    sys.path.append(view_path)
#<--[inamura 160920]

#######################################
#  CommWx
#######################################
class CommWx(object) :
    """
    CUI から起動されるCUI-GUI 通信インターフェイスクラス
    指定されたアプリケーションをプロセスとして起動し
    プロセス間通信により、コマンドを送信し
    応答を受信する。
    コマンド・及びレスポンスは'\n'をターミネータ
    とするアスキー文字列である。
    """

    #########################################
    def NewComm(self, portno, args):
        """
        args[0] で指定されたバイソンモジュールを別プロセス
        として起動しソケットにより通信接続する。
        @param  portno ソケット通信用のポート番号
        @param  args   起動モジュール名と引数(文字列)のタプル
        @retval True: 起動と通信接続に成功　　False: 失敗
        """
        fpath = args[0]

        #[inamura 160920]-->
        target_path = SearchPath(args[0])
        if target_path==None:
            return False
        fpath=target_path[1]
        #<--[inamura 160920]

        # 起動文字列作成
        strPort = "%d" % portno
        cmd = ["python", fpath, strPort]

        # 引数文字列付加
        for arg in args:
            cmd.append(arg)

        self.plot=subprocess.Popen(cmd)

        #try:
        #    # アプリケーションを別プロセスとして起動
        #    self.plot=subprocess.Popen(cmd)
        #except:
        #    print "\nError!!  Failed to create a new process."
        #    return False

        # ソケットクライアントの準備
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        # タイムアウトを 13 秒に設定
        sock.settimeout(13.0)
        # 起動待ち
        time.sleep(1.0)
        # プラットフォームが Mac なら
        if sys.platform == "darwin":
            # 起動待ち時間を追加。不具合により、Mac では一度接続に失敗すると
            # リトライによる再接続ができないため
            time.sleep(9.0)
        # 接続(リトライ10回)
        for i in range(10):
            time.sleep(3.0)
            try:
                # ローカルホストのサーバと接続
                r = sock.connect(('127.0.0.1', portno))
            except socket.error, e:
                # タイムアウト 3×10 秒
                if i == 9:
                    print '\nError: %s' % e
                    return False
            # 接続成功
            else:
                break

        # スレッド間のコマンド送受信用のキューを準備
        self.cmdQueue = Queue.Queue()
        self.ansQueue = Queue.Queue()
        # 通信サブスレッドスタート
        thin = threading.Thread(target=self.SocketClient, args=(sock,))
        thin.setDaemon(True)
        thin.start()
        # 新アプリケーション起動・接続が成功
        return True

    #############################################
    def SocketClient(self, sock) :
        """
        起動したプロセスへのコマンド送信と応答の受信
        (サブスレッドにて実行)
        @param  sock　ソケットオブジェクト
        @retval 無し
        """
        # 永久ループ
        while True:
            # 起動したアプリケーションが生きているか
            if self.plot.poll()==None:
                # キューからコマンドを読み出す
                try:
                    cmd = self.cmdQueue.get(timeout = 3.0)
                except:
                    try:
                        # 接続維持コマンド送信(3秒毎)
                        sock.send("keepalive\n")
                    except socket.error, e:
                        break
                else:
                    ans = "NACK"
                    try:
                        # アプリケーションへコマンド送信
                        sock.send(cmd)
                        # 返信読み込み
                        ans = self.GetRespons(sock)
                    except socket.error, e:
                        print '\n%s' % e

                    # 応答キューに書き込み
                    self.ansQueue.put(ans)

            # 起動したアプリケーションが終了したなら
            else:
                # アプリケーション終了による通信スレッド終了
                break
        try:
            # アプリケーション終了メッセージ
            print "\nApplication End"
            # ソケットをクローズ
            sock.close()
        except:
            # Python インタープリタ終了時に例外が発生することがあるため
            pass

    #############################################
    def GetRespons(self, sock) :
        """
        レスポンスを取得
        @param  sock ソケットオブジェクト
        @retval 無し
        """
        # レスポンスを受信するまで待っている
        while True:
            ans = ""
            # レスポンスが分割された場合のプロテクト
            for i in range(10):
                # レスポンス読み込み
                try:
                    data = sock.recv(1024)
                except:
                    return "NACK"
                # 改行コードがあるか
                if '\n' in data:
                    # 改行コードの前までをコマンドとして取り出す
                    ans = ans + data[:data.find('\n')]
                    # 受信したレスポンスを返す
                    return ans

                # レスポンスが未完であれば
                elif len(data) > 0:
                    # 連結
                    ans = ans + data
                    time.sleep(0.1)


    #########################################
    def IsAliveComm(self):
        """
        アプリケーションの生存確認
        @param  無し
        @retval True: 生存、False: 消滅
        """
        # アプリケーションが生きているか
        if self.plot.poll()==None:
            return True
        # 既に閉じられている
        else:
            return False

    #########################################
    def PutCommand(self, strCmd):
        """
        コマンドを通信スレッドへ送信(キューに書き込む)
        (本関数はメインスレッドで実行)
        @param  strCmd 通信コマンド文字列
        @retval コマンドに対する応答
        """
        # アプリケーションが生きているか
        if self.plot.poll()==None:
            # 送信キューへ書き込み
            self.cmdQueue.put(strCmd)

            # アプリケーションからの返信待ち(タイムアウト15秒)
            try:
                # キューからコマンドを読み出す
                answer = self.ansQueue.get(timeout = 15.0)
            except:
                # 15秒以内に応答が無かったので
                answer = "NACK"
            return answer
        # 既に閉じられている
        else:
            # 終了エラー
            print "The aplication has been already closed."
            return None #[inamura 170227]


if __name__ == '__main__':
    SliceViewer()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import utsusemi.DNA.ana.Reduction.AbsorptionCorrection   as ACR
import utsusemi.DNA.ana.Reduction.EfficiencyCorrection   as EFC
import utsusemi.DNA.ana.Reduction.EnergyOriginCorrection as EOC
import utsusemi.DNA.ana.Reduction.PylabPlotTools         as PPT
import utsusemi.DNA.ana.Reduction.SearchPathDNA          as SPD
import os
# special reserved word for commands history
HISTORY = None

# reserved words for return value in this.
DAT = None
ecs = None
EC  = None
ret = None
ECA = None
ECM = None
ECM1 = None
ECM2 = None
VAL  = None
###################################################################################################
# Energy Origin Correction
###################################################################################################
def MovingAverage(dat=DAT,sample=2):
    """
    MovingAverage
    @retval ECM     (ElementContainerMatrix)
    """
    ECM = EOC.MovingAverage(dat,sample)
    return ECM
#--------------------------------------------------------------------------------------------------
def CalcTOFOffset(dat=ECM, energyRange="-0.1,0.1",energyLimit="-1.0,-1.0",peakLimit="-1.0,-1.0",
                       outputFilename="tofOffset.eorg", failedFilename="failedData.txt"):
    """
    CalcTOFOffset
    @retval None
    """
    try:
        energyMin, energyMax = energyRange.split(",")
        energyMin      = float(energyMin)
        energyMax      = float(energyMax)
    except:
        energyMin      = float(-1.0)
        energyMax      = float(1.0)
    try:
        energyLimitMin, energyLimitMax = energyLimit.split(",")
        energyLimitMin = float(energyLimitMin)
        energyLimitMax = float(energyLimitMax)
    except:
        energyLimitMin = float(-1.0)
        energyLimitMax = float(-1.0)
    try:
        peakLimitMin, peakLimitMax     = peakLimit.split(",")
        peakLimitMin   = float(peakLimitMin)
        peakLimitMax   = float(peakLimitMax)
    except:
        peakLimitMin   = float(-1.0)
        peakLimitMax   = float(-1.0)

    #outputFilename = os.path.join(os.sep,"home","dnaadmin","ana","EOrg",outputFilename)
    #failedFilename = os.path.join(os.sep,"home","dnaadmin","ana","EOrg",failedFilename)
    #outputFilename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","EOrg",outputFilename)
    #failedFilename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","EOrg",failedFilename)
    outputFilename_dir = SPD.CheckForOutput("EOrg")
    if outputFilename!="":
        outputFilename = os.path.join( outputFilename_dir, outputFilename )
        failedFilename = os.path.join( outputFilename_dir, failedFilename )
    else:
        raise UserWarning("EneOrig::CalcTOFOffset >> EOrg dir is not found.")

    return EOC.CalcTOFOffset(dat, energyMin, energyMax, energyLimitMin, energyLimitMax,
                       peakLimitMin, peakLimitMax, outputFilename, failedFilename)
###################################################################################################
# Energy Origin Correction (assist)
###################################################################################################
def PutValArray(ec,energyMin,energyMax):
    """
    PutValArray
    """
    return EOC.PutValArray(ec,energyMin,energyMax)
#--------------------------------------------------------------------------------------------------
def EstimateFittingInitial(x,y):
    """
    EstimateFittingInitial
    """
    return EOC.EstimateFittingInitial(x,y)
#--------------------------------------------------------------------------------------------------
def ExecGaussFit(xarray,yarray,guess):
    """
    ExecGaussFit
    """
    return EOC.ExecGaussFit(xarray,yarray,guess)
#--------------------------------------------------------------------------------------------------
def GaussFunction(x,height,center,sigma,constant):
    """
    GaussFunction
    """
    return EOC.GaussFunction(x,height,center,sigma,constant)
#--------------------------------------------------------------------------------------------------
def ExecGaussFitEC(ec=EC,energyMin=-0.1,energyMax=0.1,height=-1.0,center=-1.0,sigma=-1.0,back=-1.0):
    """
    ExecGaussFitEC
    @retval DAT
    """
    DAT = EOC.ExecGaussFitEC(ec,energyMin,energyMax,height,center,sigma,back)
    return DAT
###################################################################################################
# Absorption Correction (create coefficient file)
###################################################################################################
def CalcASS(eiArray="1.5,2.0,2.5,3.0,3.5,4.0", sampleFileName="MMNi.dat", degree=4,
                                                output="out.ass", laticeSpace=3.134600, alpha=1.0):
    """
    CalcASS
    @retval None
    """
    if os.path.exists(sampleFileName):
        pass
    else:
        #sampleFileName = os.path.join(os.sep,"home","dnaadmin","ana","setup",sampleFileName)
        #sampleFileName = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sampleFileName)
        ret = SPD.CheckForInput( "setup", sampleFileName )
        if ret=="":
            raise UserWarning("CalcASS >> sampleFileName does not found (%s)"%(sampleFileName))
        else:
            sampleFileName = ret
    #output = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",output)
    #output = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",output)
    #ret = SPD.CheckForOutput( "Absorb", output )
    ret = SPD.CheckForOutput( "Absorb" )
    if ret=="":
        raise UserWarning("CalcASS >> output does not found (%s)"%(output))
    else:
        output = os.path.join(os.sep, ret, output )

    eiArray = [float(val) for val in eiArray.split(",")]

    return ACR.CalcASS(eiArray, sampleFileName ,degree, output, laticeSpace, alpha)
#--------------------------------------------------------------------------------------------------
def CalcACSC(eiArray="1.5,2.0,2.5,3.0,3.5,4.0", sampleFileName="MMNi.dat", degree=4,
                                               output="out.acsc", laticeSpace=3.134600, alpha=1.0):
    """
    CalcACSC
    @retval None
    """
    if os.path.exists(sampleFileName):
        pass
    else:
        #sampleFileName = os.path.join(os.sep,"home","dnaadmin","ana","setup",sampleFileName)
        #sampleFileName = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sampleFileName)
        ret = SPD.CheckForInput( "setup", sampleFileName )
        if ret=="":
            raise UserWarning("CalcACSC >> sampleFileName does not found (%s)"%(sampleFileName))
        else:
            sampleFileName = ret
    #output = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",output)
    #output = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",output)
    ret = SPD.CheckForOutput( "Absorb" )
    if ret=="":
        raise UserWarning("CalcACSC >> output does not found (%s)"%(output))
    else:
        output = os.path.join(os.sep, ret, output )

    eiArray = [float(val) for val in eiArray.split(",")]

    return ACR.CalcACSC(eiArray, sampleFileName ,degree, output, laticeSpace, alpha)
#--------------------------------------------------------------------------------------------------
def CalcACSS(eiArray="1.5,2.0,2.5,3.0,3.5,4.0", sampleFileName="MMNi.dat", degree=4,
                                               output="out.acss", laticeSpace=3.134600, alpha=1.0):
    """
    CalcACSS
    @retval None
    """
    if os.path.exists(sampleFileName):
        pass
    else:
        #sampleFileName = os.path.join(os.sep,"home","dnaadmin","ana","setup",sampleFileName)
        #sampleFileName = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sampleFileName)
        ret = SPD.CheckForInput( "setup", sampleFileName )
        if ret=="":
            raise UserWarning("CalcACSS >> sampleFileName does not found (%s)"%(sampleFileName))
        else:
            sampleFileName = ret
    #output = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",output)
    #output = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",output)
    ret = SPD.CheckForOutput( "Absorb" )
    if ret=="":
        raise UserWarning("CalcACSS >> output does not found (%s)"%(output))
    else:
        output = os.path.join(os.sep, ret, output )
        #output = ret

    eiArray = [float(val) for val in eiArray.split(",")]

    return ACR.CalcACSS(eiArray, sampleFileName ,degree, output, laticeSpace, alpha)
#--------------------------------------------------------------------------------------------------
def CalcACC(eiArray="1.5,2.0,2.5,3.0,3.5,4.0", sampleFileName="MMNi.dat", degree=4, 
                                                output="out.acc", laticeSpace=3.134600, alpha=1.0):
    """
    CalcACCC
    @retval None
    """
    if os.path.exists(sampleFileName):
        pass
    else:
        #sampleFileName = os.path.join(os.sep,"home","dnaadmin","ana","setup",sampleFileName)
        #sampleFileName = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sampleFileName)
        ret = SPD.CheckForInput( "setup", sampleFileName )
        if ret=="":
            raise UserWarning("CalcACC >> sampleFileName does not found (%s)"%(sampleFileName))
        else:
            sampleFileName = ret
    #output = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",output)
    #output = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",output)
    ret = SPD.CheckForOutput( "Absorb" )
    if ret=="":
        raise UserWarning("CalcACC >> output does not found (%s)"%(output))
    else:
        output = os.path.join(os.sep, ret, output )
        #output = ret

    eiArray = [float(val) for val in eiArray.split(",")]

    return ACR.CalcACC(eiArray, sampleFileName ,degree, output, laticeSpace, alpha)
#--------------------------------------------------------------------------------------------------
def CalcASSOfElastic(sampleFileName="MMNi.dat", output="out.ele", laticeSpace=3.134600, alpha=1.0):
    """
    CalcASSOfElastic
    @retval None
    """
    if os.path.exists(sampleFileName):
        pass
    else:
        #sampleFileName = os.path.join(os.sep,"home","dnaadmin","ana","setup",sampleFileName)
        #sampleFileName = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sampleFileName)
        ret = SPD.CheckForInput( "setup", sampleFileName )
        if ret=="":
            raise UserWarning("CalcASSOfElastic >> sampleFileName does not found (%s)"%(sampleFileName))
        else:
            sampleFileName = ret
    #output = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",output)
    #output = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",output)
    ret = SPD.CheckForOutput( "Absorb" )
    if ret=="":
        raise UserWarning("CalcACCOfElastic >> output does not found (%s)"%(output))
    else:
        output = os.path.join(os.sep, ret, output )
        #output = ret

    return ACR.CalcASSOfElastic(sampleFileName, output, laticeSpace, alpha)
#--------------------------------------------------------------------------------------------------
def ReadAbsorptionCoefficientFileOfElastic(filename):
    """
    ReadAbsorptionCoefficientFileOfElastic
    """

    if os.path.exists(filename):
        pass
    else:
#        filename = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",filename)
        #filename = os.path.join(os.sep,"home","dnaadmin","ana","EOrg",filename)
        #filename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","EOrg",filename)
        ret= SPD.CheckForInput("EOrg", filename)
        if ret!="":
            filename = ret
        else:
            raise UserWarning("EneOrig::ReadAbsorptionCoefficientFileOfElastic >> EOrg dir is not found.")

    return ACR.ReadAbsorptionCoefficientFileOfElastic(filename)
###################################################################################################
# Absorption Correction (assist)
###################################################################################################
def CalcAbsorptionCoeff(mode="ASS", eiArray="1.5,2.0,2.5,3.0,3.5,4.0", ef = 2.08, aa=0.26, pa=1.84,
                                                                        sampleFileName="MMNi.dat"):
    """
    CalcAbsorptionCoeff
    @retval EC
    """
    if os.path.exists(sampleFileName):
        pass
    else:
        #sampleFileName = os.path.join(os.sep,"home","dnaadmin","ana","setup",sampleFileName)
        #sampleFileName = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sampleFileName)
        ret = SPD.CheckForInput( "setup", sampleFileName )
        if ret!="":
            sampleFileName = ret
        else:
            raise UserWarning("EneOrig::CalcAbsorptionCoeff >> sampleFileName %s is not found"%(sampleFileName))

    aa = aa * 180.0/3.14
    pa = pa * 180.0/3.14

    eiArray = [float(val) for val in eiArray.split(",")]
    EC= ACR.CalcAbsorptionCoeff(mode, eiArray, ef, aa, pa, sampleFileName)
    return EC
#--------------------------------------------------------------------------------------------------
def ExecPolyFitEC(ec=DAT,degree=4):
    """
    ExecPolyFitEC
    @retval EC
    """
    EC=ACR.ExecPolyFitEC(ec,degree)
    return EC
#--------------------------------------------------------------------------------------------------
def ExecPolyFit(xarray,yarray,degree):
    return ACR.ExecPolyFit(xarray,yarray,degree)
#--------------------------------------------------------------------------------------------------
def PolyFunction(xarray,param):
    return ACR.PolyFunction(xarray,param)
###################################################################################################
# Absorption Correction (exec correction)
###################################################################################################
def DoAbsorptionCorrectionWithoutCellData(ecm=ECM,filename="input.ass"):
    """
    DoAbsorptionCorrectionWithoutCellData
    @retval DAT
    """
    if os.path.exists(filename):
        pass
    else:
        #filename = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",filename)
        #filename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",filename)
        ret = SPD.CheckForInput( "Absorb", filename )
        if ret!="":
            filename = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionWithoutCellData >> filename %s is not found"%(filename))

    DAT = ACR.DoAbsorptionCorrectionWithoutCellData(ecm,filename)
    return DAT
#-------------------------------------------------------------------------------------------------
def DoAbsorptionCorrectionCellData(ECM_I_C_C=ECM2, 
                              AcscFname="input.acsc", AccFname="input.acc", AsscFname="input.assc"):
    """
    DoAbsorptionCorrectionWithCellData
    @retval DAT
    """
    if os.path.exists(AcscFname):
        pass
    else:
        #AcscFname = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",AcscFname)
        #AcscFname = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",AcscFname)
        ret = SPD.CheckForInput( "Absorb", AcscFname )
        if ret!="":
            AcscFname = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionWithCellData >> AcscFname %s is not found"%(AcscFname))

    if os.path.exists(AccFname):
        pass
    else:
        #AccFname = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",AccFname)
        #AccFname = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",AccFname)
        ret = SPD.CheckForInput( "Absorb", AccFname )
        if ret!="":
            AccFname = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionWithCellData >> AccFname %s is not found"%(AccFname))

    if os.path.exists(AsscFname):
        pass
    else:
        #AsscFname = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",AsscFname)
        #AsscFname = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",AsscFname)
        ret = SPD.CheckForInput( "Absorb", AsscFname )
        if ret!="":
            AsscFname = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionWithCellData >> AsscFname %s is not found"%(AsscFname))

    DAT = ACR.DoAbsorptionCorrectionCellData(ECM_I_C_C,AcscFname,AccFname,AsscFname)
    return DAT
#-------------------------------------------------------------------------------------------------
def DoAbsorptionCorrectionWithCellData(ECM_I_SC_SC=ECM1, ECM_I_C_C=ECM2, 
                              AcscFname="input.acsc", AccFname="input.acc", AsscFname="input.assc"):
    """
    DoAbsorptionCorrectionWithCellData
    @retval DAT
    """
    if os.path.exists(AcscFname):
        pass
    else:
        #AcscFname = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",AcscFname)
        #AcscFname = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",AcscFname)
        ret = SPD.CheckForInput( "Absorb", AcscFname )
        if ret!="":
            AcscFname = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionWithCellData >> AcscFname %s is not found"%(AcscFname))

    if os.path.exists(AccFname):
        pass
    else:
        #AccFname = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",AccFname)
        #AccFname = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",AccFname)
        ret = SPD.CheckForInput( "Absorb", AccFname )
        if ret!="":
            AccFname = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionWithCellData >> AccFname %s is not found"%(AccFname))

    if os.path.exists(AsscFname):
        pass
    else:
        #AsscFname = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",AsscFname)
        #AsscFname = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",AsscFname)
        ret = SPD.CheckForInput( "Absorb", AsscFname )
        if ret!="":
            AsscFname = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionWithCellData >> AsscFname %s is not found"%(AsscFname))

    DAT = ACR.DoAbsorptionCorrectionWithCellData(ECM_I_SC_SC,ECM_I_C_C,AcscFname,AccFname,AsscFname)
    return DAT
#-------------------------------------------------------------------------------------------------
def DoAbsorptionCorrectionOfElastic(dat=ECM,filename="input.ele"):
    """
    DoAbsorptionCorrectionWithCellData
    @retval DAT
    """
    if os.path.exists(filename):
        pass
    else:
        #filename = os.path.join(os.sep,"home","dnaadmin","ana","Absorb",filename)
        #filename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Absorb",filename)
        ret = SPD.CheckForInput( "Absorb", filename )
        if ret!="":
            filename = ret
        else:
            raise UserWarning("EneOrig::DoAbsorptionCorrectionOfElastic >> filename %s is not found"%(filename))

    DAT = ACR.DoAbsorptionCorrectionOfElastic(dat,filename)
    return DAT
###################################################################################################
# Calc scattering cross section
###################################################################################################
def GetCrossSection(container="sample", sample_filename="MMVana.dat", nist_filename="all_nist_data.dat"):
    """
    GetCrossSection
    @retval VAL
    """
    if os.path.exists(nist_filename):
        pass
    else:
        #nist_filename = os.path.join(os.sep,"home","dnaadmin","ana","setup","database",nist_filename)
        #nist_filename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup","database",nist_filename)
        if "UTSUSEMI_USR_PRIV_HOME" in os.environ:
            nist_filename = os.path.join( os.environ["UTSUSEMI_USR_DIR"],
                                          os.environ["UTSUSEMI_SYS_NAME"],
                                          "ana","database" )
        else:
            nist_filename = os.path.join( os.environ["UTSUSEMI_BASE_DIR"],
                                          os.environ["UTSUSEMI_INST_CODE"],
                                          "ana","database" )
        if not os.path.exists( nist_filename ):
            raise UserWarning("Nist file does not found (%s)"%(nist_filename))

    if os.path.exists(sample_filename):
        pass
    else:
        #sample_filename = os.path.join(os.sep,"home","dnaadmin","ana","setup",sample_filename)
        #sample_filename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sample_filename)
        sample_filename = SPD.CheckForInput( "setup",sample_filename )
        if sample_filename=="":
            raise UserWarning("sample file does not found (%s)"%(sample_filename))

    VAL=EFC.GetCrossSection(container, sample_filename, nist_filename)
    print("cross section :",VAL)
    return VAL
#-------------------------------------------------------------------------------------------------
def GetCrossSectionScaleFactor(dat=ECM, energyMin=-0.01, energyMax=0.01,
                               container1="sample", sample_filename1="MMVana.dat", nist_filename="all_nist_data.dat"):
    """
    GetCrossSection
    @retval VAL
    """
    if os.path.exists(nist_filename):
        pass
    else:
        #nist_filename = os.path.join(os.sep,"home","dnaadmin","ana","setup","database",nist_filename)
        #nist_filename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup","database",nist_filename)
        if "UTSUSEMI_USR_PRIV_HOME" in os.environ:
            nist_filename = os.path.join( os.environ["UTSUSEMI_USR_DIR"],
                                          os.environ["UTSUSEMI_SYS_NAME"],
                                          "ana","database" )
        else:
            nist_filename = os.path.join( os.environ["UTSUSEMI_BASE_DIR"],
                                          os.environ["UTSUSEMI_INST_CODE"],
                                          "ana","database" )
        if not os.path.exists( nist_filename ):
            raise UserWarning("Nist file does not found (%s)"%(nist_filename))

    if os.path.exists(sample_filename1):
        pass
    else:
        #sample_filename1 = os.path.join(os.sep,"home","dnaadmin","ana","setup",sample_filename1)
        #sample_filename1 = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","setup",sample_filename1)
        sample_filename = SPD.CheckForInput( "setup",sample_filename )
        if sample_filename=="":
            raise UserWarning("sample file does not found (%s)"%(sample_filename))

    VAL = EFC.PutCrossSectionScaleFactor(dat, energyMin, energyMax,
                                         container1, sample_filename1, nist_filename)
    print("coss section scale factor :",VAL)
    return VAL
#-------------------------------------------------------------------------------------------------
def GetAverage(dat=ECM, energyMin=-0.01, energyMax=0.01):
    """
    GetAverage
    @retval VAL
    """
    VAL = EFC.PutAverage(dat, energyMin, energyMax)
    return VAL
###################################################################################################
# Efficiency Correction
###################################################################################################
def CreateMaskFileFromAverageIntensity(dat=ECM, energyMin=-0.01, energyMax=0.01, output="newMask.txt", thresh=0.5):
    """
    CreateMaskFileFromAverageIntensity
    @retval None
    """
    #output = os.path.join(os.sep,"home","dnaadmin","ana","Mask",output) # Modify [Matsuura 20150414]
    #output = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Mask",output)
    ret = SPD.CheckForOutput( "Mask" )
    if ret=="":
        raise UserWarning("CreateMaskFileFromAverageIntensity >> Not found Mask directory")
    else:
        output = os.path.join(os.sep, ret, output )
    return EFC.CreateMaskFileFromAverageIntensity(dat,energyMin,energyMax,output,thresh)
#-------------------------------------------------------------------------------------------------
def CreateEfficiencyFile(dat=ECM, energyMin=-0.01, energyMax=0.01, output="newEff.txt"):
    """
    CreateEfficiencyFile
    @retval None
    """
    #output = os.path.join(os.sep,"home","dnaadmin","ana","Efficiency",output)
    #output = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Efficiency",output)
    ret = SPD.CheckForOutput( "Efficiency" )
    if ret=="":
        raise UserWarning("CreateEfficiencyFile >> Not found Efficiency directory" )
    else:
        output = os.path.join(os.sep,ret,output)
    return EFC.CreateEfficiencyFile(dat,energyMin,energyMax,output)
#-------------------------------------------------------------------------------------------------
def DoEfficiencyCorrection(dat=ECM,filename="Eff.txt",mode=1):
    """
    DoEfficiencyCorrection
    @retval DAT
    """
    #filename = os.path.join(os.sep,"home","dnaadmin","ana","Efficiency",filename)
    #filename = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Efficiency",filename)
    ret = SPD.CheckForOutput( "Efficiency" )
    if ret=="":
        raise UserWarning("DoEfficiencyCorrection >> Not found Efficiency directory")
    else:
        filename = os.path.join(os.sep, ret, filename)
    DAT = EFC.DoEfficiencyCorrection(dat,filename,mode)
    return DAT
#-------------------------------------------------------------------------------------------------
def DoCrossSectionScaleFactorCorrection(dat=ECM,scalefactor=1.0,sampleMass=1.0,unitMass=1.0):
    """
    DoCrossSectionScaleFactorCorrection
    @retval DAT
    """
    DAT = EFC.DoCrossSectionScaleFactorCorrection(dat,scalefactor,sampleMass,unitMass)
    return DAT
#-------------------------------------------------------------------------------------------------
def DeleteMask_DirectBeam(input="Mask.txt", output="newMask.txt"):
    """
    DeleteMask_DirectBeam
    @retval None
    """
    #filename1 = os.path.join(os.sep,"home","dnaadmin","ana","Mask",input)
    #filename2 = os.path.join(os.sep,"home","dnaadmin","ana","Mask",output)
    #filename1 = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Mask",input)
    #filename2 = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Mask",output)
    filename1 = ""
    filename2 = ""
    ret = SPD.CheckForInput( "Mask", input )
    if ret=="":
        raise UserWarning("DeleteMask_DirectBeam >> Not found input file (%s)"%(input))
    else:
        filename1 = ret

    ret = SPD.CheckForOutput( "Mask" )
    if ret=="":
        raise UserWarning("DeleteMask_DirectBeam >> Not found Mask directory")
    else:
        filename2 = os.path.join(os.sep, ret, output )
    return EFC.DeleteMask_DirectBeam(filename1,filename2)
#-------------------------------------------------------------------------------------------------
def DeleteMask_Spurious(input="Mask.txt", output="newMask.txt"):
    """
    DeleteMask_Spurious
    @retval None
    """
    #filename1 = os.path.join(os.sep,"home","dnaadmin","ana","Mask",input)
    #filename2 = os.path.join(os.sep,"home","dnaadmin","ana","Mask",output)
    #filename1 = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Mask",input)
    #filename2 = os.path.join(os.sep,os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","Mask",output)
    filename1 = ""
    filename2 = ""
    ret = SPD.CheckForInput( "Mask", input )
    if ret=="":
        raise UserWarning("DeleteMask_Spurious >> Not found input file (%s)"%(input))
    else:
        filename1 = ret

    ret = SPD.CheckForOutput( "Mask" )
    if ret=="":
        raise UserWarning("DeleteMask_Spurious >> Not found Mask directory")
    else:
        filename2 = os.path.join(os.sep, ret, output )
    return EFC.DeleteMask_Spurious(filename1,filename2)

###################################################################################################
# Pylab Plot Tools
###################################################################################################
def EcmIntensity2TwoLayerArray(dat):
    return PPT.EcmIntensity2TwoLayerArray(dat)
#-------------------------------------------------------------------------------------------------
def PylabPlotTwoLayerArray(varray, xregion, yregion, zregion):
    return PPT.PylabPlotTwoLayerArray(varray, xregion, yregion, zregion)

#########################################
# Dictionary for entry the name of functions
_functions = {"MovingAverage":MovingAverage,
              "CalcTOFOffset":CalcTOFOffset,
              "ExecGaussFitEC":ExecGaussFitEC,
              "CalcASS":CalcASS,
              "CalcACSC":CalcACSC,
              "CalcACSS":CalcACSS,
              "CalcACC":CalcACC,
              "CalcASSOfElastic":CalcASSOfElastic,
              "CalcAbsorptionCoeff":CalcAbsorptionCoeff,
              "ExecPolyFitEC":ExecPolyFitEC,
              "DoAbsorptionCorrectionWithoutCellData":DoAbsorptionCorrectionWithoutCellData,
              "DoAbsorptionCorrectionWithCellData":DoAbsorptionCorrectionWithCellData,
              "DoAbsorptionCorrectionOfElastic":DoAbsorptionCorrectionOfElastic,
              "DoAbsorptionCorrectionCellData":DoAbsorptionCorrectionCellData, 
              "GetCrossSection":GetCrossSection,
              "GetCrossSectionScaleFactor":GetCrossSectionScaleFactor,
              "GetAverage":GetAverage,
              "CreateMaskFileFromAverageIntensity":CreateMaskFileFromAverageIntensity,
              "CreateEfficiencyFile":CreateEfficiencyFile,
              "DoEfficiencyCorrection":DoEfficiencyCorrection,
              "DoCrossSectionScaleFactorCorrection":DoCrossSectionScaleFactorCorrection,
              "DeleteMask_DirectBeam":DeleteMask_DirectBeam,
              "DeleteMask_Spurious":DeleteMask_Spurious
             }



import numpy
#-------------------------------------------------------------------------------------------------
def EcmIntensity2TwoLayerArray(dat):
    xn = dat.PutTableSize()
    yn = dat.Put(0).PutTableSize()
    ecxy=[]
    for i in range(xn):
        eca=dat.PutPointer(i)
        ecy=[]
        for j in range(yn):
            ec=eca.PutPointer(j)
            yv=numpy.array(ec.PutYList())
            ev=numpy.array(ec.PutEList())
            sumval=numpy.sum(yv[ev>0])
            ecy.append(sumval)
        ecxy.append(ecy)
    v=numpy.array(ecxy)
    return v
#-------------------------------------------------------------------------------------------------
def PylabPlotTwoLayerArray(varray, xregion, yregion, zregion):

    import pylab

    hv   = len(varray)/2
    v_up = varray[:hv]
    v_up = numpy.array(v_up)
    v_up = numpy.transpose(v_up)

    v_down = varray[hv:]
    v_down = [ ve[-1::-1] for ve in v_down[-1::-1] ]
    v_down = numpy.array(v_down)
    v_down = numpy.transpose(v_down)

    Z = numpy.vstack((v_up,v_down))

    pylab.figure(figsize=(10,4))

    #x = numpy.linspace(0, len(Z[0]), len(Z[0])+1)
    #y = numpy.linspace(0, len(Z), len(Z)+1)
    #X, Y = numpy.meshgrid(x, y)
    #pylab.pcolor(X, Y, Z)
    xc = numpy.linspace(0, len(Z[0]), len(Z[0]))
    yc = numpy.linspace(0, len(Z), len(Z))

    minval=zregion[0]
    maxval=zregion[1]

    div   =(maxval-minval)/1000.0
    pylab.contourf(xc,yc,Z,numpy.arange(minval,maxval,div),extend="both")

    pylab.xlabel("Psd No.")
    pylab.ylabel("Pixel No.")

    pylab.colorbar()
    pylab.xlim(xregion)
    pylab.ylim(yregion)

    pylab.show()

